@@ -595,11 +595,11 @@ class CartImageCube(ImageCube):
 
         if action_fct is None:
             if fmhz == 'med':
-                fmhz = self.freqs[0] + (self.freqs[-1] - self.freqs[0]) / 2.
+                fmhz = (self.freqs[0] + (self.freqs[-1] - self.freqs[0]) / 2.) * 1e-6
             elif fmhz == 'first':
-                fmhz = self.freqs[0]
+                fmhz = self.freqs[0] * 1e-6
             elif fmhz == 'last':
-                fmhz = self.freqs[-1]
+                fmhz = self.freqs[-1] * 1e-6
 
             i = np.argmin(abs(self.freqs - fmhz * 1e6))
             d = d[i]
