@@ -175,6 +175,64 @@ class BetaVAE(nn.Module):
         return self.decode(mu)
 
 
+class PreProcessor(object):
+    
+    def process(self, ps_data, n_sigma_clip=3):
+        data_sum = ps_data.sum(axis=1)
+        mask_zero = ~(data_sum < 1e-20)
+        print('Discarding PS with zero power:', (~mask_zero).sum())
+        
+        data_sum_log = np.log10(data_sum[mask_zero])
+        data_sum_log[~np.isfinite(data_sum_log)] = -1e5
+        mask = astats.sigma_clip(data_sum_log, sigma=3, stdfunc=psutil.robust_std).mask
+        print('Discarding outliers PS:', (mask).sum())
+        
+        ps_data = ps_data[mask_zero][~mask]
+
+        return ps_data / ps_data.sum(axis=1)[:, None]
+
+    def inv(self, ps):
+        return ps
+
+
+class PreProcessorFlatten(PreProcessor):
+    
+    def __init__(self, k_mean, alpha=1):
+        self.k_mean = k_mean
+        self.alpha = alpha
+
+    def process(self, ps_data, n_sigma_clip=3):
+        ps_data = PreProcessor.process(self, ps_data, n_sigma_clip=n_sigma_clip)
+
+        ps_data = ps_data / self.k_mean ** self.alpha
+        norm_factor = ps_data.max()
+        ps_data = ps_data / norm_factor
+        
+        return ps_data
+        
+    def inv(self, ps):
+        return self.k_mean * ps
+
+
+class PreProcessorLogScale(PreProcessor):
+    
+    def __init__(self, k_mean):
+        self.k_mean = k_mean
+        self.norm_factor = 1
+
+    def process(self, ps_data, n_sigma_clip=3):
+        ps_data = PreProcessor.process(self, ps_data, n_sigma_clip=n_sigma_clip)
+
+        ps_data = np.log10(ps_data)
+        self.norm_factor = (- ps_data).max()
+        ps_data = - ps_data / self.norm_factor
+        
+        return ps_data
+        
+    def inv(self, ps):
+        return 10 ** (-self.norm_factor * ps)
+
+
 class AbstractFitter(object):
 
     def __init__(self, n_dim, k_mean):
@@ -303,6 +361,35 @@ class PCAFitter(AbstractFitter):
         return self.pca.inverse_transform(self.pca.transform(data))
 
 
+class VAEFitterPreProc(VAEFitter):
+
+    def __init__(self, model, optimizer, pre_proc):
+        self.model = model
+        self.optimizer = optimizer
+        self.loss = []
+        self.val_loss = []
+        self.all_rec_loss = []
+        self.pre_proc = pre_proc
+        AbstractFitter.__init__(self, model.latent_dim, pre_proc.k_mean)
+
+    def train(self, epochs, train_data, frac_validation=0.1, batch_size=128):
+        train_data = self.pre_proc.process(train_data.copy())
+
+        np.random.shuffle(train_data)
+
+        i = int(train_data.shape[0] * (1 - frac_validation))
+        val_data = train_data[i:]
+        train_data = train_data[:i]
+        print(f'Training set: {train_data.shape[0]}, Validation set: {val_data.shape[0]}')
+
+        train_loader = DataLoader(train_data, batch_size=batch_size, shuffle=True)
+        val_loader = DataLoader(val_data, batch_size=batch_size, shuffle=True)
+
+        VAEFitter.train(self, epochs, train_loader, val_loader)
+        
+        return FitterResult(self, train_loader, val_loader)
+
+
 class FitterResult(AbstractFitter):
 
     def __init__(self, fitter, train_data, test_data):
@@ -811,6 +898,10 @@ class VAEKernTorch(AbstractMLKern, MultiKern):
             and the k_mean of the traing sets (ps3d_k_mean).'''
         self.fitter_res = vae_fitter
         self.ps3d_k_mean = vae_fitter.k_mean
+        if not isinstance(self.fitter_res, VAEFitterPreProc):
+            self.pre_proc = PreProcessorFlatten(self.ps3d_k_mean)
+        else:
+            self.pre_proc = self.fitter_res.pre_proc
         AbstractMLKern.__init__(self, vae_fitter.n_dim, name=name, param_values=param_values)
 
     def copy(self):
@@ -821,8 +912,7 @@ class VAEKernTorch(AbstractMLKern, MultiKern):
 
     def get_norm_cov_1d(self, freqs, params):
         cov_gen = get_cached_cov_gen(tuple(freqs), tuple(self.uv_bins))
-        ps3d = self.fitter_res.decode(params).squeeze()
-        ps3d = self.ps3d_k_mean ** 1 * ps3d
+        ps3d = self.pre_proc.inv(self.fitter_res.decode(params).squeeze())
 
         return cov_gen.c_nu_from_ps21(self.ps3d_k_mean, ps3d)
 
@@ -871,7 +961,7 @@ class MultiStationaryKern(MultiKern):
     ''' extension of kernel for regression in multiple baselines range'''
 
     def __init__(self, kern_class, variance=1, lengthscale=1, ls_alpha=0, var_alpha=0, theta_rad=0.1, name='mkern',
-                 wedge_parametrization=False, ls_is_period=False):
+                 wedge_parametrization=False, ls_is_period=False, latitude_deg=90):
         MultiKern.__init__(self)
         self.kern_class = kern_class
         self.kerns = []
@@ -879,6 +969,7 @@ class MultiStationaryKern(MultiKern):
         
         self.wedge_parametrization = wedge_parametrization
         self.ls_is_period = ls_is_period
+        self.latitude_deg = latitude_deg
 
         self.var_alpha = GPy.core.parameterization.Param('var_alpha', var_alpha)
         self.var_alpha.constrain_fixed(var_alpha)
@@ -896,9 +987,12 @@ class MultiStationaryKern(MultiKern):
     @classmethod
     def load_from_dict(cls, name, d):
         wedge_parametrization = False
+        latitude_deg = 90
         if 'wedge_parametrization' in d:
             wedge_parametrization = d['wedge_parametrization']
-        kern = cls(name=name, wedge_parametrization=wedge_parametrization)
+        if 'latitude_deg' in d:
+            latitude_deg = d['latitude_deg']
+        kern = cls(name=name, wedge_parametrization=wedge_parametrization, latitude_deg=latitude_deg)
         return MultiKern._parse_set_params_from_dict(kern, d)
 
     def set_var_alpha_prior(self, lower, upper):
@@ -943,7 +1037,7 @@ class MultiStationaryKern(MultiKern):
         var_norm = 1 / np.mean((u_means / u_min) ** self.var_alpha * u_means / u_means.mean())
         for i, u_mean in enumerate(u_means):
             if self.wedge_parametrization:
-                delays = np.sin(self.theta_rad) * u_mean / (self.mean_fmhz)
+                delays = (np.sin(self.theta_rad) + np.cos(np.radians(self.latitude_deg))) * u_mean / (self.mean_fmhz)
             else:
                 delays = (1 + self.ls_alpha * 1e-3 * l_m * (u_mean - u_min)) / l_m
             ll = 1 / delays
