@@ -115,8 +115,14 @@ def even_odd_to_sum_diff(even, odd, sum, diff):
 
     even = datacube.CartDataCube.load(even)
     odd = datacube.CartDataCube.load(odd)
-    (0.5 * (even + odd)).save(sum)
-    (0.5 * (even - odd)).save(diff)
+    
+    sum_c = (0.5 * (even + odd))
+    sum_c.weights = 2 * sum_c.weights
+    sum_c.save(sum)
+
+    diff_c = (0.5 * (even - odd))
+    diff_c.weights = 2 * diff_c.weights
+    diff_c.save(diff)
 
     print('All done')
 
